/*****************************************************************************/
/** File    : confirm.c                                                     **/
/** Purpose : create the confirmation window                                **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/

#include "xproc.h"

#include <X11/Shell.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/List.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Command.h> 

#include "Xedw/XedwForm.h"

#include "interface.h"
#include "structure.h"

extern void Realize_Center (Widget); /* center.c */
extern struct icon_name * Goto_struct (dqueue,String); /* goto.c */
extern void Ok_Choice (Widget, XtPointer, XtPointer); /* actions.c */
extern int Nb_struct (dqueue); /* list.c */
extern int highlightedPsList (String *); /* list.c */
extern void * Our_XtMalloc(long); /* main.c */
extern void End_connect(int);

extern int sock;

void Confirm (String ConfirmLabel, String commandName,
	      void *ExecuteFunction, void *CancelFunction)
{
  /* popup confirm window and raise callback 
     ExecuteFunction if ok
     CancelFunction else */

  Arg arglist[4];
  int i,lenght;
  int params=0;
  String label;
  String * items;

  w_confirm.itself = 
    XtCreatePopupShell ("Confirm",
			overrideShellWidgetClass,
			w_main.itself,
			NULL, ZERO);

  w_confirm.main_form = 
    XtCreateManagedWidget ("Main_form",
			   xedwFormWidgetClass,
			   w_confirm.itself,
			   NULL, ZERO);
  
  lenght=((commandName==NULL)?0:
	  strlen((Goto_struct (config_list[Command],commandName))->function));
  
  w_confirm.label.name = (String)Our_XtMalloc (sizeof (char)
				   *(strlen ("Do You Really Want to \n\n\n  ")
				     +strlen (ConfirmLabel)
				     +lenght));
  
  sprintf (w_confirm.label.name, 
	   "\nDo You Really Want to %s %s\n\n\0",
	   ConfirmLabel,
	   (commandName==NULL)?"":
	   (Goto_struct (config_list[Command],commandName))->function);
  
  XtSetArg (arglist[0], XtNlabel, w_confirm.label.name); 
  
  w_confirm.label.itself = 
    XtCreateManagedWidget ("Main_label",
			   labelWidgetClass,
			   w_confirm.main_form,
			   arglist, 1);
  
  if (commandName != NULL)
    { /* display all selected processes */
      params=Goto_struct(config_list[Command],commandName)->parameters;
      if(params!=0)
	{
	  w_confirm.list_form.list.ps = (String *) Our_XtMalloc (sizeof(String)*100);
	  w_confirm.list_form.list.number = highlightedPsList (w_confirm.list_form.list.ps);
	  
	  if (w_confirm.list_form.list.number!=0)
	    {
	      XtSetArg (arglist[0], XtNfromVert, w_confirm.label.itself); 
	      XtSetArg (arglist[1], XtNresizable,True);
	      w_confirm.list_form.itself =  
		XtCreateManagedWidget ("Ps_form",
				       formWidgetClass,
				       w_confirm.main_form,
				       arglist, 2);

	      w_confirm.list_form.scroll = 
		XtCreateManagedWidget ("Scroll",
				       viewportWidgetClass,
				       w_confirm.list_form.itself,
				       NULL,0);
	      
	      XtSetArg (arglist[0], XtNlist, w_confirm.list_form.list.ps); 
	      XtSetArg (arglist[1], XtNnumberStrings, w_confirm.list_form.list.number); 
	      XtSetArg (arglist[2], XtNfromVert, w_confirm.label.itself); 
	      
	      w_confirm.list_form.list.itself = 
		XtCreateManagedWidget ("List",
				       listWidgetClass,
				       w_confirm.list_form.scroll,
				       arglist, 3);
	    }
	  else
	    XtFree(w_confirm.list_form.list.ps);
	}
      else 
	{ /* display all host name to choice */
	  items=(String *)Our_XtMalloc(sizeof(String)*
				       Nb_struct(config_list[Host]));
	  dq_go_beginning(config_list[Host]);
	  i=0;
	  while(!dq_at_end(config_list[Host]))
	    {
	      items[i]=XtNewString(((struct icon_name *)dq_get(config_list[Host]))->name);
	      i++;
	      dq_go_forward(config_list[Host]);
	    }
	  items[i]=NULL;
	  XtSetArg (arglist[0], XtNfromVert, w_confirm.label.itself); 
	  
	  w_confirm.choice.form = 
	    XtCreateManagedWidget("Host_form",
				  xedwFormWidgetClass,
				  w_confirm.main_form,
				  arglist,1);
	  
	  w_confirm.choice.label =
	    XtCreateManagedWidget("Label",
				  labelWidgetClass,
				  w_confirm.choice.form,
				  NULL,ZERO);
	  
	  XtSetArg(arglist[0], XtNlist, items);
	  XtSetArg(arglist[1], XtNfromVert, w_confirm.choice.label);
	  
	  w_confirm.choice.list = 
	    XtCreateManagedWidget("List",
				  listWidgetClass,
				  w_confirm.choice.form,
				  arglist, 2);
	  
	  XtAddCallback(w_confirm.choice.list, XtNcallback, Ok_Choice,NULL);
	}
    }  
  
  if(params==0&&commandName!=NULL)
    XtSetArg (arglist[0], XtNfromVert, w_confirm.choice.form);
  else
    if (commandName==NULL||w_confirm.list_form.list.number==0)
      XtSetArg (arglist[0], XtNfromVert, w_confirm.label.itself);
    else
      XtSetArg (arglist[0], XtNfromVert, w_confirm.list_form.itself);
  
  
  w_confirm.answer_form.itself = 
    XtCreateManagedWidget ("Answer_form",
			   formWidgetClass,
			   w_confirm.main_form,
			   arglist, 1);
  
  
  w_confirm.answer_form.execute = 
    XtCreateManagedWidget ("Execute",
			   commandWidgetClass,
			   w_confirm.answer_form.itself,
			   NULL, ZERO);
  
  XtSetArg (arglist[0], XtNfromHoriz, w_confirm.answer_form.execute); 
  XtSetArg (arglist[1], XtNwidthLinked, w_confirm.answer_form.execute);
  
  w_confirm.answer_form.cancel = 
    XtCreateManagedWidget ("Cancel",
			   commandWidgetClass,
			   w_confirm.answer_form.itself,
			   arglist, 2);
  
  XtAddCallback (w_confirm.answer_form.cancel, XtNcallback, CancelFunction, NULL);
  XtAddCallback (w_confirm.answer_form.execute, XtNcallback, ExecuteFunction, commandName);
  
  Realize_Center (w_confirm.itself);
}

void Quitquit (Widget w, XtPointer client_data, XtPointer call_data)
{
  End_connect(sock);
  XtDestroyApplicationContext (app_con);
  exit (0);
}

void Quitcancel (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtPopdown (w_confirm.itself);
}

void Quit (Widget w, XtPointer client_data, XtPointer call_data)
{
  Confirm ("Quit?", NULL, Quitquit, Quitcancel);
}
