/*
  File: 	queue.h
  Author:	Robert Strandh
  Purpose:
  	Header file for double ended queue package.
*/

#ifndef _DQ_PACKAGE_
/* data structures */

struct dq_cell_structure
{
  void *element;
  struct dq_cell_structure *prev;
  struct dq_cell_structure *next;
};

typedef struct dq_cell_structure *dq_cell;

typedef struct queue_structure *dqueue;

struct queue_structure
{
  dq_cell head;
  dq_cell tail;
  dq_cell cursor;
  dqueue cursor_save;
  void (*element_destroy)();
};

/* The queue uses the concept of a cursor. A cursor works like the emacs
   dot, i.e. it is located between two element, before the first element,
   or after the last element.

   Every queue has a cursor stack. The top of the stack is the cursor
   used by the queue operations. We call it the current cursor. */

/* create an empty queue */
extern dqueue dq_create();
/* check whether queue is empty */
extern int  dq_empty(dqueue);
/* destroy the queue */
extern void dq_destroy(dqueue);
/* set the destroy function for individual elements */
extern void dq_set_destroy(dqueue, void (*)());

/* functions to position the current cursor /*
/* position current cursor before the first element */
extern void dq_go_beginning(dqueue);
/* position current cursor after the last element */
extern void dq_go_end(dqueue);
/* move current cursor forward */
extern void dq_go_forward(dqueue);
/* move current cursor backward */
extern void dq_go_backward(dqueue);

/* functions for checking where the current cursor is located */
/* check if current cursor is located before the first element */
extern int  dq_at_beginning(dqueue);
/* check if current cursor is located after the last element */
extern int  dq_at_end(dqueue);

/* get element after current cursor without moving current cursor */
extern void *dq_get(dqueue);
/* get first element of queue without moving current cursor */
extern void *dq_get_first(dqueue);


/* functions for insertion and deletion of elements */
/* insert an element after current cursor and position the curren cursor
   after the newly inserted element */
extern void dq_insert(dqueue, void *);
/* delete element after the current cursor */
extern void dq_delete(dqueue);
/* replace the element after the current cursor */
extern void dq_replace(dqueue, void *);

/* functions for manipulating the cursor stack */
/* push old cursor and create a new current cursor with undefined value */
extern void dq_push_cursor(dqueue);
/* pop the cursor stack replacing the current cursor with value pushed */
extern void dq_pop_cursor(dqueue);

extern dqueue dq_copy(dqueue);

/* clearing a queue (added by the xproc group) */
extern void dq_clear(dqueue);

#define _DQ_PACKAGE_
#endif _DQ_PACKAGE_
