%{
/*****************************************************************************/
/** File    : give_command.l                                                **/
/** Purpose : Slit up a command line in argument list recognized by exec    **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/

#include <stdio.h>
#include "defines.h"
#include "structure.h"
#include "dqueue.h"
#define  EOS    0
#define  STRING 1

char *argument;
int find_pid=0;

%}

delim		[ \t]
char		[^ ^']
string		{char}+ 

%START QUOTE

%%
<QUOTE>{delim}*   ;
<QUOTE>"$"PID     {sprintf(argument,"%s%s ",argument,"%d");
                   find_pid=1;
                  }
<QUOTE>{string}   sprintf(argument,"%s%s ",argument,yytext);
<QUOTE>\'         {BEGIN 0;
                   sprintf(yytext,"%s",argument);
                   return (STRING);
                  }
{delim}*          {/* no action */}
[\n]              {return (EOS);}
"$"PID            {sprintf(yytext,"%s","%d");
                   find_pid=1;
                   return (STRING);
                  }                   
\'                {BEGIN QUOTE;
                   argument=(char *)malloc(sizeof(char));
                   *argument='\0';
                  }
{string}          {return (STRING);}

%%

int yywrap(void) 
{
  /* function called when EOF encounterd */

  return 1;
}

char ** Give_command(int * number) 
{
  /* Used to call the lexical analyser */

  int token;
  char **arg_value;

  arg_value=(char **)Our_alloc(MAX_SIZE);
  *number=0;
  while((token=yylex()) != EOS) {
    arg_value[*number]=(char *)Our_alloc(strlen(yytext)+1);
    strcpy(arg_value[*number],yytext);
    if (find_pid)
      find_pid=*number;
    *number=(*number)+1;
  }
  return(arg_value);
} 


void Exec_command(char * command,dqueue list_ps) 
{
  /* Execute 'command' on the processes in 'list_ps'. */

  char ** arg, *string , *temp;
  int i=0,lex_pipe[2],*argc,argv=0;

  string=(char *)Our_alloc(strlen(command)+3);
  argc=(int *)Our_alloc(sizeof(int));
  sprintf(string,"%s \n ",command);

  /* The command line is given to LEX by a pipe */
  if (pipe(lex_pipe) == -1) {
    fprintf(stderr,"Error : pipe not created in Exec_command().\n");
    exit(1);
  }
  write(lex_pipe[1],string,strlen(string));
  close(0);
  dup(lex_pipe[0]);

  arg=Give_command(argc); 
  /* arg contains a argument list corresponding with the command line */

  close(lex_pipe[0]);

  /* The command is executed for all process identifier give in the list_ps */
  dq_go_beginning(list_ps);
  while(!dq_at_end(list_ps)) {
    argv=1;
    if (fork() == 0) {
      if (find_pid!=0) {
        /* the process list must be treated */
        temp=(char *)Our_alloc(strlen(arg[find_pid]));
        strcpy(temp,arg[find_pid]);
        sprintf(arg[find_pid],temp,((struct ps *)dq_get(list_ps))->pid);
      }
      else {
        /* the exec command must be applied without process identifier */
        dq_go_end(list_ps);
        dq_go_backward(list_ps);
      }
      execvp(arg[0],arg);
      exit(1);
    }
    else {
      wait(0);
      dq_go_forward(list_ps);
    }
  }
  if (!argv) 
    execvp(arg[0],arg);
}



 












