/*****************************************************************************/
/** File    : init_host.c                                                   **/
/** Purpose : Find processes in proc zone and fill structure using u zone   **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/user.h>
#include <sys/proc.h>
#include <fcntl.h>
#include <kvm.h>
#include "structure.h"
#include "defines.h"
#include "dqueue.h"

extern void *Our_alloc(long);  /* From our_alloc.c */


struct host_ps * Init_structure_host(int uid)
{ 
  /* Search the processes of 'uid' in the proc zone and fill the structure
     using the u zone. */

  struct proc *proc_zone;
  struct user *u_zone;
  struct host_ps *structure_host;
  struct ps *cell_ps;
  kvm_t *kernel;
  int proc_nb=0;
  char ***arg;
  dqueue ps_list;

  structure_host=(struct host_ps *)Our_alloc(sizeof(struct host_ps));
  structure_host->name=(char *)Our_alloc(MAXHOSTNAMELEN);
  proc_zone=(struct proc *)Our_alloc(sizeof(struct proc));
  u_zone=(struct user *)Our_alloc(sizeof(struct user));

  gethostname(structure_host->name,MAXHOSTNAMELEN);
  ps_list=dq_create();
  kernel=kvm_open("/vmunix",NULL,NULL,O_RDONLY,NULL);
  if (kernel == NULL) {
    fprintf(stderr,"Error while opening Kernel.\n");
    exit(1);
  }
  proc_zone=kvm_nextproc(kernel);
  while (1){
    if ((proc_zone == NULL) || (proc_nb >= MAXUPRC)){
      kvm_close(kernel);
      break;
    }
    if (proc_zone->p_uid == uid){
      u_zone=kvm_getu(kernel,proc_zone);
      if ((u_zone!=NULL) && (*u_zone->u_comm)) {

	/* We have to get this process. */
	cell_ps=(struct ps *)Our_alloc(sizeof(struct ps));
	cell_ps->name=(char *)Our_alloc(strlen(u_zone->u_comm)+1);

	strcpy(cell_ps->name,u_zone->u_comm);
	proc_nb++;
	arg=(char ***)Our_alloc(MAX_SIZE);

	/* kvm_getcmd is used to get the arguments of the process. */
	kvm_getcmd(kernel,proc_zone,u_zone,arg,NULL);
	if (*arg)
	  cell_ps->arg=++(*arg);
	else {

	  /* Trouble with the arguments of this process. */
	  cell_ps->arg=(char **)Our_alloc(MAX_SIZE);
	  cell_ps->arg[0]=NULL;
	}
	cell_ps->pid=proc_zone->p_pid;
	cell_ps->ppid=proc_zone->p_ppid;
	cell_ps->state=((proc_zone->p_stat == SSLEEP) &&
			(proc_zone->p_slptime > 20))?SIDL:proc_zone->p_stat;
	dq_insert(ps_list,cell_ps);
      }
    }
    proc_zone=kvm_nextproc(kernel);
  }
  dq_go_beginning(ps_list);
  structure_host->list_ps=ps_list;
  return structure_host;
}

