/*****************************************************************************/
/** File    : state.c                                                       **/
/** Purpose : create widget including all process informations              **/
/** Authors : M.L. Drillaud V. Lataste L. Pringot T. Ramiere J.L. Richard   **/
/** Date    : 1st Mar 1992                                                  **/
/*****************************************************************************/

#include "xproc.h"

#include <sys/proc.h>
#include <sys/param.h>
#include <sys/user.h>

#include <X11/Shell.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>

#include <X11/IntrinsicP.h>
#include "Xedw/XedwListP.h"     

#include "interface.h"
#include "structure.h"

#include "Xedw/XedwList.h"

extern struct host_ps * Goto_host (String  ); /* goto.c */
extern void * Our_XtMalloc(long); /* main.c */
extern int nb_w_ps;

String * Children_of(String,short);
struct ps * Extract_pid(String ,String );
String  Convert_state(short);
String * Children_of(String ,short);

void State (Widget w, XButtonEvent* event)
{  
  /* display a popup window under processes window including 
     all the parameters of the process icon selected */
  
  Arg arglist[3];
  Position x,y;
  int xtop,w_ps_i=0,i=0;
  XtTranslations translations;
  XedwListReturnStruct *tmp_list;
  struct ps * process=NULL;
  String  name;
  String * sons;

  while(w_ps_i<nb_w_ps) /* search the window_type of w */
    {
      if (w_ps[w_ps_i].list.itself==w) break;
      w_ps_i++;
    }

  XtSetArg (arglist[0], XtNtitle,&name); /* get title of w */
  XtGetValues (w_ps[w_ps_i].toplevel, arglist, 1);

  /* move state window over processes window to x/y mouse */
  XtTranslateCoords (w_ps[w_ps_i].toplevel,
                     (Position) (event->x),
                     (Position) (event->y),
                     &x, &y);


  y+=(int)((struct _WidgetRec *)w->core.y); /* WARNING: this instruction use
					       IntrinsicP.h and XedwListP.h */

  XtSetArg (arglist[0], XtNx,x);
  XtSetArg (arglist[1], XtNy,y);
    
  state.toplevel = 
    XtCreatePopupShell ("State",
			overrideShellWidgetClass,
			w_ps[w_ps_i].itself,
			arglist, 2);

  state.form.itself= 
    XtCreateManagedWidget ("Form",
			   formWidgetClass,
			   state.toplevel,
			   NULL,ZERO);

  tmp_list=XedwListShowCurrent (w);
  while(tmp_list->next!=NULL)
    {
      tmp_list=tmp_list->next; /* search the last selected item */
    }
  if(tmp_list->xedwList_index!=-1)
    process=Extract_pid(name,tmp_list->string); 
  
  if(process==NULL) 
    {
      state.form.name.label= XtNewString("No Process Selected\n");
      XtSetArg (arglist[0], XtNlabel, state.form.name.label);
      state.form.name.itself = 
	XtCreateManagedWidget (NULL,
			       labelWidgetClass,
			       state.form.itself,
			       arglist, 1);
    }
  else
    {
      /* Name */
      state.form.name.label= (String)Our_XtMalloc (sizeof(char)*
						   (strlen(process->name)+
						    strlen("\nname:\n")));
      sprintf (state.form.name.label,"\nname:%s\n",process->name);
      XtSetArg (arglist[0], XtNlabel, state.form.name.label);
      state.form.name.itself = 
	XtCreateManagedWidget (NULL,
			       labelWidgetClass,
			       state.form.itself,
			       arglist, 1);
      
      if((process->arg[0])!=NULL)
	{
	  /* Args */
	  state.form.args.label=(String)Our_XtMalloc(sizeof(char)*MAXARGSLEN);
	  sprintf(state.form.args.label,"args:\n");
	  i=0;
	  while((process->arg[i])!=NULL)
	    {
	      sprintf(state.form.args.label,"%s%s\n",
		      state.form.args.label,process->arg[i]); 
	      i++;
	    }
	  XtSetArg (arglist[0], XtNlabel, state.form.args.label);
	  XtSetArg (arglist[1], XtNfromVert,state.form.name.itself);
	  state.form.args.itself = 
	    XtCreateManagedWidget (NULL,
				   labelWidgetClass,
				   state.form.itself,
				   arglist, 2);
	}
      /* PID */
      state.form.pid.label=(String)Our_XtMalloc((sizeof(char)*
						 strlen("pid:\n"))+
				    sizeof(int));
      sprintf (state.form.pid.label,"pid:%d\n",process->pid);
      XtSetArg (arglist[0], XtNlabel, state.form.pid.label);
      XtSetArg (arglist[1], XtNfromVert,
		((process->arg[0])!=NULL )?
		state.form.args.itself:state.form.name.itself);
      state.form.pid.itself = 
	XtCreateManagedWidget (NULL,
			       labelWidgetClass,
			       state.form.itself,
			       arglist, 2);
      /* State */      
      state.form.state.label=(String)Our_XtMalloc(sizeof(char)*
						  (strlen("state:\n")+
						   strlen(Convert_state
							  (process->state))));
      sprintf (state.form.state.label,"state:%s\n",
	       Convert_state(process->state));
      XtSetArg (arglist[0], XtNlabel, state.form.state.label);
      XtSetArg (arglist[1], XtNfromVert,state.form.pid.itself);
      state.form.state.itself = 
	XtCreateManagedWidget (NULL,
			       labelWidgetClass,
			       state.form.itself,
			       arglist, 2);
      /* PPID */
      state.form.ppid.label=(String)Our_XtMalloc((sizeof(char)*
						  strlen("ppid:\n"))+
						 sizeof(int));
      sprintf (state.form.ppid.label,"ppid:%d\n",process->ppid);

      XtSetArg (arglist[0], XtNlabel, state.form.ppid.label);
      XtSetArg (arglist[1], XtNfromVert,state.form.state.itself);
      state.form.ppid.itself = 
	XtCreateManagedWidget (NULL,
			       labelWidgetClass,
			       state.form.itself,
			       arglist, 2);
      sons=Children_of(name,process->pid);     

     
      if(sons[0]!=NULL)
	{
	  /* CHILDREN */
	  state.form.child.label= (String)Our_XtMalloc((sizeof(char)*
							strlen("children\n"))+
						       (sizeof(char)*
							MAXCOMLEN*MAXUPRC));
	  sprintf(state.form.child.label,"children:\n");
	  i=0;
	  while(sons[i]!=NULL)
	    {
	      sprintf(state.form.child.label,"%s%s\n",
		      state.form.child.label,sons[i]);
	      i++;
	    }
	  XtSetArg (arglist[0], XtNlabel, state.form.child.label);
	  XtSetArg (arglist[1], XtNfromVert,state.form.ppid.itself);
	  state.form.child.itself = 
	    XtCreateManagedWidget (NULL,
				   labelWidgetClass,
				   state.form.itself,
				   arglist, 2);
	}
    }
  
  XtRealizeWidget (state.toplevel);
  XtPopupSpringLoaded(state.toplevel);
}

String * Children_of(String  host,short ppid)
{
  /* search all processes which have ppid as PPID 
     and return char ** with "children_i(pid_i)" */
     
  dqueue process;
  String * children;
  int i=0;
  
  children=(String *)Our_XtMalloc(sizeof(String )*MAXUPRC);
  
  process=(Goto_host(host))->list_ps;
  dq_go_beginning(process);
  while(!dq_at_end(process))
    {
      if(((struct ps *)dq_get(process))->ppid==ppid)
	{
	  children[i]=(String )Our_XtMalloc(sizeof(char)*
				       (strlen
					(((struct ps *)dq_get
					  (process))->name)+25));
	  sprintf(children[i],"%s(%d)",
		  ((struct ps *)dq_get(process))->name,
		  ((struct ps *)dq_get(process))->pid);
	  i++;
	}
      dq_go_forward(process);
    }
  children[i]=NULL;
  return children;
}

String Convert_state(short state)
{
  switch(state) {
  case SSLEEP:
    return "Sleeping";
    break;
  case SWAIT:
    return "Waiting";
    break;
  case SRUN:
    return "Running";
    break;
  case SIDL:
    return "Idle";
    break;
  case SZOMB:
    return "Zombie";
    break;
  case SSTOP:
    return "Stopped";
    break;
  }
}

struct ps * Extract_pid(String  host_name,String  displayed_ps)
{
    /* extract pid in ps_name(pid) 
       and return the process structure with PID=pid */

  dqueue process;
  int i=0,j=0;
  String  pid;
  
  pid= (String)Our_XtMalloc(sizeof(char)*strlen(displayed_ps));

  while(displayed_ps[i]!='\0' && displayed_ps[i]!='(' )
    i++;
  i++; /* pass '(' */
  while(displayed_ps[i]!='\0' && displayed_ps[i]!=')' )
    {
      pid[j]=displayed_ps[i];
      i++;j++;
    }
  pid[j]='\0';

  process=(Goto_host(host_name))->list_ps;

  /* search process in host: host_name
     with PID: pid (PID is unique) */
  dq_go_beginning(process);
  while(!dq_at_end(process)&&
	((struct ps *)dq_get(process))->pid!=atoi(pid))
    dq_go_forward(process);

  XtFree(pid);
  return (struct ps *)dq_get(process);
}

void UnState (Widget w, XButtonEvent* event)
{
  /* popdown widget state and free memory */
  XtFree(state.form.name.label);
  XtFree(state.form.pid.label);
  XtFree(state.form.state.label);
  XtFree(state.form.ppid.label);
  XtFree(state.form.args.label);
  XtFree(state.form.child.label);
  XtPopdown(state.toplevel);
}

 









